/**
 * Author: Claudia D'Ambrosio
 * Date: 2012/02/14
 * Main for lecture
 */

#if defined(_MSC_VER)
// Turn off compiler warning about long names
#  pragma warning(disable:4786)
#endif
#include <iomanip>
#include <fstream>

#include "CoinTime.hpp"
#include "CoinError.hpp"

#include "BonOsiTMINLPInterface.hpp"
#include "BonIpoptSolver.hpp"
#include "MyTMINLP.hpp"
#include "MySolution.hpp"
#include "BonCbc.hpp"
#include "BonBonminSetup.hpp"

#include "BonOACutGenerator2.hpp"
#include "BonEcpCuts.hpp"
#include "BonOaNlpOptim.hpp"
//#define REDIRECT

int main (int argc, char *argv[])
{
  WindowsErrorPopupBlocker();

  int n_config = 3;

  using namespace Ipopt;
  using namespace Bonmin;
  SmartPtr<MyTMINLP> tminlp = new MyTMINLP;
  
#ifdef REDIRECT
  FILE * fp = fopen("log.out","w");
  CoinMessageHandler handler(fp);
  BonminSetup bonmin(&handler);
#else
  BonminSetup bonmin;
#endif
  bonmin.initializeOptionsAndJournalist();
  
  bonmin.readOptionsFile();// This reads the default file "bonmin.opt"
 
  //Now initialize from tminlp
  bonmin.initialize(GetRawPtr(tminlp));


  //Set up done, now let's branch and bound
  double time1;
  vector<MySolution> solutions;

  int iter;
  double of_temp;
  int status_temp;

  for(iter=0; iter<n_config; iter++)
  {
    time1 = CoinCpuTime();

    if(iter==0)
      bonmin.readOptionsString("bonmin.algorithm B-BB\n");
    if(iter==1)
      bonmin.readOptionsString("bonmin.num_resolve_at_root 5\n");
    if(iter==2)
    {
      bonmin.readOptionsString("bonmin.cutoff_decr -100\n");
      bonmin.readOptionsString("bonmin.allowable_gap -100\n");
      bonmin.readOptionsString("bonmin.allowable_fraction_gap -100\n");
    }
    bonmin.initialize(GetRawPtr(tminlp));

    try {
      Bab bb;
      bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      of_temp = bb.bestObj();
      status_temp = bb.mipStatus();
      std::cout << "Iteration " << iter;
      std::cout << ". Best solution: " << of_temp;
      std::cout << ". Status: " << status_temp << std::endl;
      solutions.push_back(MySolution(status_temp,of_temp));

    }
    catch(TNLPSolver::UnsolvedError *E) {
      //There has been a failure to solve a problem with Ipopt.
      std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
    }
    catch(OsiTMINLPInterface::SimpleError &E) {
      std::cerr<<E.className()<<"::"<<E.methodName()
  	     <<std::endl
	     <<E.message()<<std::endl;
    }
    catch(CoinError &E) {
      std::cerr<<E.className()<<"::"<<E.methodName()
	     <<std::endl
	     <<E.message()<<std::endl;
    }
  }


  return 0;
}

